using System.Text.Json;
using System.Text.Json.Serialization;
using System.Text.Json.Nodes;
using System.Threading.Tasks;

namespace Example;

public class TestServiceClient {

   private JRpcClient Client;

   public TestServiceClient(JRpcClient client) {
      this.Client = client;
   }

   public async Task<AddValueResponse?> AddValuesSingleParam(AddValueRequest request) {
      var param = new JsonArray(JsonSerializer.SerializeToNode(request));
      return await this.Client.SendRequest<AddValueResponse>("TestService.AddValuesSingleParam", param);
   }
   
   public async Task<double?> AddValuesMultipleParams(double value1, double value2) {
      var param = new JsonArray(JsonSerializer.SerializeToNode(value1), JsonSerializer.SerializeToNode(value2));
      return await this.Client.SendRequest<double>("TestService.AddValuesMultipleParams", param);
   }
   
   public async Task ReturningVoid(double param1) {
      var param = new JsonArray(JsonSerializer.SerializeToNode(param1));
      await this.Client.SendRequestRaw("TestService.ReturningVoid", param);
   }
   
   public void OnEvent(string param1) {
      var param = new JsonArray(JsonSerializer.SerializeToNode(param1));
      this.Client.SendNotification("TestService.OnEvent", param);
   }
   
   public async Task ThrowingError() {
      var param = new JsonArray();
      await this.Client.SendRequestRaw("TestService.ThrowingError", param);
   }
   
   public async Task<IList<double>?> FunctionWithArrayAsParamAndReturn(List<double> values1, List<double> values2) {
      var param = new JsonArray(JsonSerializer.SerializeToNode(values1), JsonSerializer.SerializeToNode(values2));
      return await this.Client.SendRequest<IList<double>>("TestService.FunctionWithArrayAsParamAndReturn", param);
   }
   
}